<?php

declare(strict_types=1);

namespace Drupal\Tests\nodepermissions\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\user\Entity\Role;
use Drupal\user\RoleInterface;

/**
 * Confirm that this module's permissions have the intended effects.
 *
 * @group nodepermissions
 */
final class FieldAccessTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['node', 'nodepermissions'];

  /**
   * Admin user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $adminUser;

  /**
   * Permissions created by this module.
   *
   * The keys are the name attributes on the node-edit form.`
   *
   * @var string[]
   */
  protected array $permissions = [
    'uid[0][target_id]' => 'administer node uid',
    'status[value]' => 'administer node status',
    'created[0][value][date]' => 'administer node created',
    'promote[value]' => 'administer node promote',
    'sticky[value]' => 'administer node sticky',
  ];

  /**
   * ID of the test node.
   *
   * @var int
   */
  protected int $nid;

  /**
   * Role for the test user.
   *
   * @var \Drupal\user\RoleInterface
   */
  protected RoleInterface $role;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->createContentType(['type' => 'page']);
    $this->nid = (int) $this->createNode()->id();

    $this->role = Role::load(
      $this->createRole(['edit any page content']),
    );
    $this->adminUser = $this->drupalCreateUser();
    $this->adminUser->addRole($this->role)->save();
    $this->drupalLogin($this->adminUser);

  }

  /**
   * Test callback.
   */
  public function testFieldAccess(): void {
    // Add each permission, one at a time, to the role.
    $this->assertFieldsEditable();
    foreach ($this->permissions as $permission) {
      $this->role
        ->grantPermission($permission)
        ->trustData()
        ->save();
      $this->assertFieldsEditable();
    }
    // Revoke each permission, one at a time, from the role.
    foreach ($this->permissions as $permission) {
      $this->role
        ->revokePermission($permission)
        ->trustData()
        ->save();
      $this->assertFieldsEditable();
    }
  }

  /**
   * Asserts that exactly the expected fields can be edited.
   */
  protected function assertFieldsEditable() {
    $this->drupalGet("/node/{$this->nid}/edit");
    $this->assertSession()->statusCodeEquals(200);
    foreach ($this->permissions as $name => $permission) {
      $selector = sprintf('//input[@name = "%s"]', $name);
      if ($this->adminUser->hasPermission($permission)) {
        $this->assertSession()->elementExists('xpath', $selector);
      }
      else {
        $this->assertSession()->elementNotExists('xpath', $selector);
      }
    }
  }

}
